package zero1hd.rhythmbullet.audio.visualizer;

import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;

import zero1hd.rhythmbullet.audio.MusicController;

public class DoubleHorizontalVisualizer {
	private int width, height, barCount, barWidth, spaceBetweenBars;
	private int x, y;
	private float barRate = 0.75f;
	private ShapeRenderer shapeRenderer;
	private PCMSystem pcm;
	private int smoothRange;
	private int multiplier = 300;
	private int[] amplitudes;
	private int[] barHeights;
	private int binsPerBar;
	
	/**
	 * 
	 * @param barCount amount of bars this visualizer should have.
	 * @param width the width of the visualizer.
	 * @param spacePercentage the percentage of a bar that should be space.
	 */
	public DoubleHorizontalVisualizer(int barCount, int width, int height, float spacePercentage, MusicController musicController, PCMSystem PCMSystem) {
		this.barCount = barCount;
		this.barWidth = width/barCount;
		this.spaceBetweenBars = (int) (barWidth * spacePercentage);
		this.barWidth -= spaceBetweenBars;
		pcm = PCMSystem;
		if (barWidth < 1) throw new IllegalArgumentException("The arguments you passed caused the bar width to be 0.");
		binsPerBar = (pcm.getFrequencyBins().length/barCount);
		this.width = width;
		this.height = height;
		amplitudes = new int[barCount];
		barHeights = new int[barCount];
		shapeRenderer = new ShapeRenderer();
	}
	
	public void act(float delta) {
		for (int bar = 0; bar < amplitudes.length; bar++) {
			float normalizedAmplitude = 0;
			for (int freq = bar*binsPerBar; freq < (bar*binsPerBar) + binsPerBar; freq++) {
				normalizedAmplitude += Math.abs(pcm.getFrequencyBins()[freq]);
			}
			amplitudes[bar] = (int) (normalizedAmplitude*multiplier);
			amplitudes[bar] /= binsPerBar;
			
			barHeights[bar] += Math.max(0, (amplitudes[bar] - barHeights[bar]) * barRate * delta);
			if (barHeights[bar] > amplitudes[bar]) barHeights[bar] = amplitudes[bar];
			
		}
		for (int bar = 0; bar < barHeights.length; bar++) {
			int smoothCount = 1;
			for (int range = 0; range < smoothRange; range++) {
				if (bar+range < amplitudes.length) {
					barHeights[bar] += amplitudes[bar+range];
					smoothCount++;
				}
				if (bar-range > 0) {
					barHeights[bar] += amplitudes[bar-range];
					smoothCount++;
				}
			}
			barHeights[bar] /= smoothCount;
		}
	}
	
	public void draw(Batch batch, float parentAlpha) {
		batch.end();
		shapeRenderer.begin(ShapeType.Filled);
		shapeRenderer.setColor(1f, 1f, 1f, 1f);
		shapeRenderer.setProjectionMatrix(batch.getProjectionMatrix());
		shapeRenderer.setTransformMatrix(batch.getTransformMatrix());
		shapeRenderer.rect(x, y, width, 2);
		shapeRenderer.rect(x, y+height-2, width, 2);
		int beginX = x + spaceBetweenBars/2, beginY = y; 
		for (int bar = 0; bar < barCount; bar++) {
			shapeRenderer.rect(beginX + (spaceBetweenBars+barWidth)*bar, beginY+height, barWidth, barHeights[bar]);
			shapeRenderer.rect(beginX + (spaceBetweenBars+barWidth)*bar, beginY-barHeights[barHeights.length - 1 - bar], barWidth, barHeights[barHeights.length - 1 - bar]);
		}
		shapeRenderer.end();
		batch.begin();
	}
	
	public int getWidth() {
		return width;
	}
	
	public int getHeight() {
		return height;
	}
	
	public void setX(int x) {
		this.x = x;
	}
	
	public void setY(int y) {
		this.y = y;
	}
	
	public void setPosition(int x, int y) {
		this.x = x;
		this.y = y;
	}
	
	public int getX() {
		return x;
	}
	
	public int getY() {
		return y;
	}
}
