package zero1hd.rhythmbullet.desktop.screens.main;

import java.util.Observable;
import java.util.Observer;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;
import com.badlogic.gdx.utils.Align;

import zero1hd.rhythmbullet.RhythmBullet;
import zero1hd.rhythmbullet.audio.MusicController;
import zero1hd.rhythmbullet.audio.visualizer.DoubleHorizontalVisualizer;
import zero1hd.rhythmbullet.desktop.audio.visualizer.PCMObtainer;
import zero1hd.rhythmbullet.desktop.graphics.ui.components.MusicControls;
import zero1hd.rhythmbullet.desktop.graphics.ui.pages.Page;
import zero1hd.rhythmbullet.graphics.ui.components.ScrollText;

public class MainPage extends Page implements Observer {
	private MusicController mc;
	
	private Label versionLabel;
	private Image title;
	
	private Table menuTable;
	private TextButton playButton;
	private TextButton optionsButton;
	private TextButton quitButton;
	
	private MusicControls musicControls;
	private ScrollText scrollText;
	
	private DoubleHorizontalVisualizer dhv;
	
	public MainPage(MusicController musicController, AssetManager assetManager, Skin skin, ChangeListener playButtonListener, ChangeListener optionsButtonListener) {
		super(0, 0);
		this.mc = musicController;
		this.mc.addObserver(this);
		
		dhv = new DoubleHorizontalVisualizer(70, (int) getWidth(), (int) (getHeight()*0.3), 0.3f, mc, new PCMObtainer(mc));
		dhv.setPosition(0, (int) ((getHeight() - dhv.getHeight())/2f));
		
		title = new Image(assetManager.get("title.png", Texture.class));
		title.setScale((dhv.getHeight() - 14)/title.getHeight());
		if (title.getWidth()*title.getScaleX() > dhv.getWidth() - 30) {
			title.setScale((dhv.getWidth() - 30)/title.getWidth()*getScaleX());
		}
		title.setPosition((getWidth()-title.getWidth()*title.getScaleX())/2f, (getHeight()-title.getHeight()*title.getScaleY())/2f);
		addActor(title);
		
		versionLabel = new Label("Version: " + RhythmBullet.VERSION, skin, "sub-font", skin.getColor("default"));
		versionLabel.setPosition(3, 3);
		addActor(versionLabel);
		
		menuTable = new Table();
		menuTable.setSize(getWidth(), dhv.getY());
		menuTable.align(Align.center);
		menuTable.defaults().space(10f);
		addActor(menuTable);
		playButton = new TextButton("Start!", skin);
		playButton.addListener(playButtonListener);
		menuTable.add(playButton).width(Gdx.graphics.getWidth()*0.2f);
		
		menuTable.row();
		
		optionsButton = new TextButton("Options", skin, "sub");
		optionsButton.addListener(optionsButtonListener);
		menuTable.add(optionsButton).fillX();
		
		menuTable.row();
		
		quitButton = new TextButton("Quit", skin, "sub");
		quitButton.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				Gdx.app.exit();
			}
		});
		menuTable.add(quitButton).fillX();
		
		musicControls = new MusicControls(skin, mc);
		musicControls.setPosition((getWidth()-musicControls.getMinWidth() - 20f), getHeight()-musicControls.getMinHeight()-20f);
		addActor(musicControls);
		
		scrollText = new ScrollText("...", "...", skin, false, true);
		scrollText.setWidth(0.5f*getWidth());
		scrollText.setPosition(15, getHeight() - scrollText.getHeight()-25f);
		addActor(scrollText);
		
	}
	
	@Override
	public void act(float delta) {
		dhv.act(delta);
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		dhv.draw(batch, parentAlpha);
		super.draw(batch, parentAlpha);
	}

	@Override
	public void dispose() {
		super.dispose();
	}

	@Override
	public void update(Observable o, Object arg) {
		if (o == mc) {
			scrollText.setText("Currently playing: " + mc.getCurrentSongName(), null);
	    }
	}
}
